      SUBROUTINE GVSTCP (TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- SHIFTS ALL CHARACTERS LEFTWARDS IN TEXT UNTIL NO BLANKS ARE LEFT.     ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT       NONE
      CHARACTER*(*)  TEXT
      INTEGER        GVFLEN, LENTXT, ICHR, ILOOP
C----
C---- SKIP EMPTY TEXT
C----
      IF (GVFLEN(TEXT).LE.1) RETURN
C----
C---- SET THE START LENGTH
C----
      LENTXT=GVFLEN(TEXT)
C----
C---- CHECK IF THERE ARE ANY CHARACTERS
C----
      IF (LENTXT.LT.2) RETURN
C----
C---- START WITH A LEFT SHIFT
C----
      CALL GVSSHL (TEXT)
C----
C---- LOOP OVER TEXT AND FIND BLANKS
C----
      ICHR=1
   10 LENTXT=GVFLEN(TEXT)
      IF (LENTXT.LT.2) RETURN
C----
C---- LOOP OVER THE STRING TILL FIRST BLANK IS FOUND, THEN SHIFT DOWN REST
C----
      DO 30 ICHR=1,LENTXT-1
         IF (TEXT(ICHR:ICHR).EQ.' ') THEN
C----------
C---------- SHIFT ALL HIGHER CHARS ONE DOWN, AND REDO THE TEST
C----------
            DO 20 ILOOP=ICHR,LENTXT-1
               TEXT(ILOOP:ILOOP)=TEXT(ILOOP+1:ILOOP+1)
   20       CONTINUE
            TEXT(LENTXT:LENTXT)=' '
            GOTO 10
         END IF
   30 CONTINUE

      END

      INTEGER FUNCTION GVFLEN(TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- RETURNS THE LENGTH OF TEXT                                            ----
C---- THE LENGTH IS DEFINED AS THE SEQUENCE NUMBER IN TEXT OF THE LAST      ----
C---- NON-BLANK CHARACTER. UNDEFINED CHARACTERS ARE ASSUMED TO BE BLANK.    ----
C----                                                                       ----
C---- TEXT IS A CHARACTER STRING OF ANY LENGTH                              ----
C---- DO NOT FORGET TO DECLARE GVFLEN AS AN INTEGER IN THE CALLING ROUTINE  ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT         NONE
      CHARACTER*(*)    TEXT
      INTEGER          I, ICH
C----
C---- INITIATE
C----
      GVFLEN=0
C----
C---- MEASURE THE LENGTH
C----
      DO 10 I=LEN(TEXT),1,-1
         ICH=ICHAR(TEXT(I:I))
         IF (ICH.NE.ICHAR(' ').AND.ICH.NE.0) THEN
            GVFLEN=I
            RETURN
         END IF
   10 CONTINUE

      RETURN
      END
      SUBROUTINE GVSSHR (TEXT,LENGTH)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A RIGHT SHIFT IS PERFORMED ON TEXT UNTIL ALL TRAILING BLANKS ARE GONE ----
C---- TEXT SHOULD BE A CHARACTER STRING OF LENGTH LENGTH.                   ----
C----                                                                       ----
C---- STRANGE ERRORS WILL OCCUR IF TEXT IS NOT PROPERLY DEFINED IN THE      ----
C---- CALLING MODULE. CHANCES ARE THAT THESE ERRORS SHOW UP IN A TOTALLY    ----
C---- DIFFERENT WAY THEN YOU WOULD EXPECT.                                  ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT      NONE
      INTEGER       LENGTH, I
      CHARACTER*(*) TEXT
C----
C---- LOOK FOR TRIVIAL ERRORS
C----
      IF (LENGTH.LT.0) THEN
         WRITE(0,*) ('Length of input text is < zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.0) THEN
         WRITE(0,*) ('Length of input text is zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.1) THEN
         WRITE(0,*) ('Length of input text is 1 in GVSSHR')
         RETURN
      END IF
C----
C---- IF THE TEXT IS OK, SEE IF IT ENDS AT BLANK
C----
   10 IF (TEXT(LENGTH:LENGTH).NE.' ') RETURN
C----
C---- IF THERE ARE TRAILING BLANKS, SHIFT EVERYTHING UP BY ONE, AND TEST AGAIN
C----
      DO 20 I=LENGTH,2,-1
         TEXT(I:I)=TEXT(I-1:I-1)
   20 CONTINUE
      TEXT(1:1)=' '
      GOTO 10

      END

      SUBROUTINE GVSSHL (TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A LEFT SHIFT IS PERFORMED ON TEXT UNTIL ALL LEADING BLANCKS ARE GONE  ----
C---- TEXT SHOULD BE A CHARACTER STRING OF UNLIMITTED LENGTH                ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT      NONE
      CHARACTER*(*) TEXT
      INTEGER       GVFLEN, I, J, LENTXT
C----
C---- DETERMINE THE LENGTH OF THE TEXT
C----
      LENTXT=GVFLEN(TEXT)
      IF (LENTXT.GT.0) THEN
C-------
C------- IF THERE ARE LEADING BLANKS, SHIFT TO THE LEFT
C-------
         DO 30 I=1,LENTXT
            IF (ICHAR(TEXT(I:I)).NE.ICHAR(' ')) THEN
               IF (I.EQ.1) RETURN
               DO 10 J=I,LENTXT
                  TEXT(J-I+1:J-I+1)=TEXT(J:J)
   10          CONTINUE
               DO 20 J=LENTXT-I+2,LENTXT
                  TEXT(J:J)=' '
   20          CONTINUE
               RETURN
            END IF
   30    CONTINUE
      END IF

      RETURN
      END

      LOGICAL FUNCTION GVFISF(FILNAM)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- BECOMES TRUE IF THE FILE FILNAM EXISTS.                               ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT          NONE
      CHARACTER*(*)  FILNAM
      INTEGER        GVFLEN
C-------
C------- TEST FOR EMPTY FILE NAME
C-------
         IF (GVFLEN(FILNAM).EQ.0) THEN
            GVFISF=.FALSE.
            RETURN
         END IF
C-------
C------- SEE IF THE FILE EXISTS
C-------
         INQUIRE (FILE=FILNAM,EXIST=GVFISF)

      RETURN
      END

      SUBROUTINE GVSTUP (N, XA3, XB3, BOOLS, R, RMSE)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- GVSTUP CALCULATES THE RIGID BODY LEAST SQUARES MATRIX TO              ----
C---- ROTATE ONE SET OF COORDINATES INTO ANOTHER.                           ----
C---- ONLY THE ATOMS FOR WHICH BOOLS=.TRUE. WILL BE USED.                   ----
C---- THE MATRIX AND VECTOR ARE APPLIED TO XB. SO XB GETS CHANGED.          ----
C----                                                                       ----
C---- GIVEN A SET OF COORDINATES XB, WE WISH TO FIND THE MATRIX R AND       ----
C---- TRANSLATION VECTOR V THAT WHEN APPLIED TO COORDINATES WILL BEST FIT   ----
C---- THE COORDINATES XA. (I.E. XA = R * XB + V).  R IS A UNITARY 3 BY 3    ----
C---- RIGHT HANDED ROTATION MATRIX AND V IS THE OFFSET VECTOR. THIS IS AN   ----
C---- EXACT SOLUTION.                                                       ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT      NONE
      INTEGER       N, J, I, K
      REAL*4        XA ,XB ,R(3,3),
     +              UMAT(3,3), RMSE, XN, XASQ, XBSQ, CMB(3),
     +              XA3,XB3
      LOGICAL       BOOLS
      INTEGER       GVFCBT
      INCLUDE 'params.f'
      DIMENSION XA(3,MAXAT) ,XB(3,MAXAT), XA3(3*MAXAT), 
     +          XB3(3*MAXAT),BOOLS(MAXAT)

C----
C---- INITIATE
C----
      RMSE = 0.0
      XN = GVFCBT(BOOLS,N)
      XASQ = 0.0
      XBSQ = 0.0
      DO I=1,3
         DO J=1,3
            UMAT(I,J)=0.0
         ENDDO
      ENDDO
C----
C---- DAAN EDIT'S HERE. CONVERT XA3(3N) TO XA(3,N)
C----
      DO I=1,N
         K=3*(I-1)
         DO J=1,3
            XA(J,I)=XA3(K+J)
            XB(J,I)=XB3(K+J)
         ENDDO
      ENDDO
C----
C---- ACCUMULATE UNCORRECTED (FOR C.M.) SUMS AND SQUARES
C----
      DO 120 I = 1,3
         DO 110 J = 1,N
            IF (BOOLS(J)) THEN
               DO 100 K = 1,3
                  UMAT(I,K) = UMAT(I,K) + XA(I,J)*XB(K,J)
  100          CONTINUE
            END IF
  110    CONTINUE
  120 CONTINUE
C----
C---- FIT IT
C----
      CALL GVSSP2 (UMAT,R)
C----
C---- APPLIE THE MATRIX AND VECTOR TO THE SECOND COORDINATE SET : XB
C----
      DO 190 I=1,N
         DO 170 J=1,3
            CMB(J)=0.0
            DO 160 K=1,3
               CMB(J)=CMB(J)+R(J,K)*XB(K,I)
 160        CONTINUE
 170     CONTINUE
         DO 180 J=1,3
            XB(J,I)=CMB(J)
 180     CONTINUE
 190  CONTINUE
C----
C---- CALCULATE THE RMS ERROR. DEFINED AS MATCHED ATOM DISTANCE ERROR RMS
C----
      DO 200 I=1,N
         IF (BOOLS(I)) THEN
            DO J=1,3
               RMSE=RMSE+(XA(J,I)-XB(J,I))**2
            ENDDO
         ENDIF
  200 CONTINUE
      RMSE=SQRT(RMSE/XN)
C----
C---- DAAN EDIT'S HERE. CONVERT XB(3,N) TO XB3(3N) 
C----
      DO I=1,N
         K=3*(I-1)
         DO J=1,3
            XB3(K+J)=XB(J,I)
         ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE GVSSP2 (UMAT,R)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- ROUTINE TO GET THE EIGENVALUE ONLY AS ADAPTED FROM A PROGRAM          ----
C---- BY A.D. MCLACHAN. THIS IS A SUPPORT ROUTINE FOR GVSSUP.               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT      NONE
      INTEGER       ISIG, J
      REAL          UMAT(3,3) ,R(3,3) ,ROOT(3), B1, B2, A(3,3) ,
     +              B(3,3) ,UTR(6), T, B13, B23, B33, S
      EQUIVALENCE   (UTR(1), B(1,1))

      ISIG = 1
C----
C---- FORM USQ = (UT).U    (IN UPPER TRIANGULAR SYMMETRIC STORAGE MODE)
C----
      DO 10 J = 1,3
         T=UMAT(1,1)*UMAT(1,J)+UMAT(2,1)*UMAT(2,J)+UMAT(3,1)*UMAT(3,J)
         UTR(1+(J*J-J)/2) = T
   10 CONTINUE
      DO 20 J = 2,3
         T=UMAT(1,2)*UMAT(1,J)+UMAT(2,2)*UMAT(2,J)+UMAT(3,2)*UMAT(3,J)
         UTR(2+(J*J-J)/2) = T
   20 CONTINUE
      DO 30 J = 3,3
         T=UMAT(1,3)*UMAT(1,J)+UMAT(2,3)*UMAT(2,J)+UMAT(3,3)*UMAT(3,J)
         UTR(3+(J*J-J)/2) = T
   30 CONTINUE
C----
C---- CALCULATE EIGENVALUES AND VECTORS
C----
      CALL GVSSP3 (UTR,A)
      ROOT(1) = UTR(1)
      ROOT(2) = UTR(3)
      ROOT(3) = UTR(6)
C----
C---- SET A3 = A1 CROSS A2
C---- ROOTS ARE IN ORDER R(1) > = R(2) > = R(3) > = 0
C----
      A(1,3) = A(2,1)*A(3,2) - A(3,1)*A(2,2)
      A(2,3) = A(3,1)*A(1,2) - A(1,1)*A(3,2)
      A(3,3) = A(1,1)*A(2,2) - A(2,1)*A(1,2)
C----
C---- VECTOR SET B = U.A
C----
      B(1,1)=UMAT(1,1)*A(1,1)+UMAT(1,2)*A(2,1)+UMAT(1,3)*A(3,1)
      B(2,1)=UMAT(2,1)*A(1,1)+UMAT(2,2)*A(2,1)+UMAT(2,3)*A(3,1)
      B(3,1)=UMAT(3,1)*A(1,1)+UMAT(3,2)*A(2,1)+UMAT(3,3)*A(3,1)
      B(1,2)=UMAT(1,1)*A(1,2)+UMAT(1,2)*A(2,2)+UMAT(1,3)*A(3,2)
      B(2,2)=UMAT(2,1)*A(1,2)+UMAT(2,2)*A(2,2)+UMAT(2,3)*A(3,2)
      B(3,2)=UMAT(3,1)*A(1,2)+UMAT(3,2)*A(2,2)+UMAT(3,3)*A(3,2)
      B(1,3)=UMAT(1,1)*A(1,3)+UMAT(1,2)*A(2,3)+UMAT(1,3)*A(3,3)
      B(2,3)=UMAT(2,1)*A(1,3)+UMAT(2,2)*A(2,3)+UMAT(2,3)*A(3,3)
      B(3,3)=UMAT(3,1)*A(1,3)+UMAT(3,2)*A(2,3)+UMAT(3,3)*A(3,3)
C----
C---- NORMALIZE B1 AND B2 AND CALCULATE B3 = B1 CROSS B2
C----
      B1 = SQRT( B(1,1)*B(1,1) + B(2,1)*B(2,1) + B(3,1)*B(3,1) )
      B2 = SQRT( B(1,2)*B(1,2) + B(2,2)*B(2,2) + B(3,2)*B(3,2) )
      IF (B1 .EQ. 0.0) THEN
c         CALL GVSTTE ('?TLSB21')
         B1 = 1.0
      END IF
      IF (B2 .EQ. 0.0) THEN
c         CALL GVSTTE ('?TLSB22')
         B2 = 1.0
      END IF

      B(1,1) = B(1,1)/B1
      B(1,2) = B(1,2)/B2
      B(2,1) = B(2,1)/B1
      B(2,2) = B(2,2)/B2
      B(3,1) = B(3,1)/B1
      B(3,2) = B(3,2)/B2
C----
C---- CHECK FOR LEFT HANDED ROTATION
C----
      B13 = B(2,1)*B(3,2) - B(3,1)*B(2,2)
      B23 = B(3,1)*B(1,2) - B(1,1)*B(3,2)
      B33 = B(1,1)*B(2,2) - B(2,1)*B(1,2)
      S = B13*B(1,3) + B23*B(2,3) + B33*B(3,3)
      IF (S .LT. 0.0) ISIG = -1
      B(1,3) = B13
      B(2,3) = B23
      B(3,3) = B33
C----
C---- CALCULATE ROTATION MATRIX R
C----
      R(1,1)=B(1,1)*A(1,1)+B(1,2)*A(1,2)+B(1,3)*A(1,3)
      R(1,2)=B(1,1)*A(2,1)+B(1,2)*A(2,2)+B(1,3)*A(2,3)
      R(1,3)=B(1,1)*A(3,1)+B(1,2)*A(3,2)+B(1,3)*A(3,3)
      R(2,1)=B(2,1)*A(1,1)+B(2,2)*A(1,2)+B(2,3)*A(1,3)
      R(2,2)=B(2,1)*A(2,1)+B(2,2)*A(2,2)+B(2,3)*A(2,3)
      R(2,3)=B(2,1)*A(3,1)+B(2,2)*A(3,2)+B(2,3)*A(3,3)
      R(3,1)=B(3,1)*A(1,1)+B(3,2)*A(1,2)+B(3,3)*A(1,3)
      R(3,2)=B(3,1)*A(2,1)+B(3,2)*A(2,2)+B(3,3)*A(2,3)
      R(3,3)=B(3,1)*A(3,1)+B(3,2)*A(3,2)+B(3,3)*A(3,3)
C----
C---- RMS ERROR
C----
      ROOT(1) = SQRT( MAX(ROOT(1),0.0) )
      ROOT(2) = SQRT( MAX(ROOT(2),0.0) )
      ROOT(3) = SQRT( MAX(ROOT(3),0.0) )
C----
C---- CHANGE SIGN OF EVAL 3 IF LEFT HANDED
C----
      IF (ISIG .LT. 0) ROOT(3) = -ROOT(3)

      RETURN

      END

      SUBROUTINE GVSSP3 (A,R)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- ROUTINE TO GET THE EIGENVALUES AND EIGENVECTORS OF SYMMETRIC MATRIX   ----
C----                                                                       ----
C---- ARGUMENTS ARE :                                                       ----
C----                                                                       ----
C---- A = ORIGINAL MATRIX (SYMMETRIC), DESTROYED IN COMPUTATION.            ----
C----     THE RESULTING EIGENVALUES ARE DEVELOPED IN DIAGONAL OF MATRIX A   ----
C----     IN DESCENDING ORDER. THE EQUATION FOR THE INDICES OF A IS *       ----
C----                                                                       ----
C----      A(I,J) -> A(I+(J*J-J)/2))                                        ----
C----                                                                       ----
C---- R = RESULTANT MATRIX OF EIGENVECTORS (STORED COLUMNWISE,              ----
C----     IN SAME SEQUENCE AS EIGENVALUES)                                  ----
C----                                                                       ----
C---- THE ORIGINAL MATRIX A MUST BE REAL SYMMETRIC (STORAGE MODE = 1)       ----
C---- AND MATRIX A CANNOT BE IN THE SAME LOCATION AS MATRIX R               ----
C----                                                                       ----
C---- DIAGONALIZATION METHOD ORIGINATED BY JACOBI AND ADAPTED BY VON        ----
C---- NEUMANN FOR LARGE COMPUTERS AS FOUND IN 'MATHEMATICAL METHODS FOR     ----
C---- DIGITAL COMPUTERS', EDITED BY A. RALSTON AND H.S. WILF, JOHN WILEY    ----
C---- AND SONS, NEW YORK, 1962.                                             ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT      NONE
      INTEGER       IQ, J, IA, I, IND, L, M, MQ, LQ, LM, ILQ, IMQ,
     +              IM, IL, LL, IJ, ILR, IMR, JQ, MM, EXITCNT
      REAL          A(6) ,R(9), ANORM, ANRMX, RANGE, THR, SINX,
     +              SINX2, COSX, COSX2, SINCS, X, Y, GVFSQR
C----
C---- INITIALIZE
C----
      EXITCNT=0
      RANGE = 1.0E-6
      IQ    = -3
      DO 10 J = 1,3
         IQ = IQ+3
         IJ = IQ+1
         R(IJ) = 0.0
         IF (1.EQ.J) R(IJ) = 1.0
         IJ = IQ+2
         R(IJ) = 0.0
         IF (2.EQ.J) R(IJ) = 1.0
         IJ = IQ+3
         R(IJ) = 0.0
         IF (3.EQ.J) R(IJ) = 1.0
   10 CONTINUE
C----
C---- COMPUTE INITIAL AND FINAL NORMS (ANORM AND ANORMX)
C----
      ANORM = 0.0
      DO 25 I=1,2
         DO 20 J=I+1,3
            IA = I+(J*J-J)/2
            ANORM = ANORM+A(IA)*A(IA)
   20    CONTINUE
   25 CONTINUE
      IF (ANORM.LE.0.0) GOTO 290
      ANORM = GVFSQR(2.0*ANORM)
      ANRMX = ANORM*RANGE/3.0
C----
C---- INITIALIZE INDICATORS AND COMPUTE THRESHOLD, THR
C----
      IND = 0
      THR = ANORM
  120 THR = THR/3.0
  130 L = 1
  140 M = L+1
C----
C---- COMPUTE SIN AND COS
C----
  150 MQ = (M*M-M)/2
      LQ = (L*L-L)/2
      LM = L+MQ
      IF (ABS(A(LM)).LT.THR) GOTO 220
      IND = 1
      LL = L+LQ
      MM = M+MQ
      X = 0.5*(A(LL)-A(MM))
      Y = -(A(LM)/GVFSQR(A(LM)*A(LM)+X*X))
      IF (X.LT.0.0) Y = -Y
      SINX = Y/GVFSQR(2.0*(1.0+(GVFSQR(1.0-Y*Y))))
      SINX2 = SINX*SINX
      COSX = GVFSQR(1.0-SINX2)
      COSX2 = COSX*COSX
      SINCS = SINX*COSX
C----
C---- ROTATE L AND M COLUMNS
C----
      ILQ = 3*(L-1)
      IMQ = 3*(M-1)
      DO 210 I = 1,3
         IQ = (I*I-I)/2
         IF (I.NE.L.AND.I.NE.M) THEN
            IF (I.LT.M) THEN
               IM = I+MQ
            ELSE
               IM = M+IQ
            ENDIF
            IF (I.LT.L) THEN
               IL = I+LQ
            ELSE
               IL = L+IQ
            END IF
            X = A(IL)*COSX-A(IM)*SINX
            A(IM) = A(IL)*SINX+A(IM)*COSX
            A(IL) = X
         END IF
         ILR = ILQ+I
         IMR = IMQ+I
         X = R(ILR)*COSX-R(IMR)*SINX
         R(IMR) = R(ILR)*SINX+R(IMR)*COSX
         R(ILR) = X
  210 CONTINUE
      X = 2.0*A(LM)*SINCS
      Y = A(LL)*COSX2+A(MM)*SINX2-X
      X = A(LL)*SINX2+A(MM)*COSX2+X
      A(LM) = (A(LL)-A(MM))*SINCS+A(LM)*(COSX2-SINX2)
      A(LL) = Y
      A(MM) = X
C----
C---- TESTS FOR COMPLETION
C----
  220 EXITCNT=EXITCNT+1
      IF (EXITCNT.GT.1000) RETURN
      IF (M.NE.3) THEN
         M = M+1
         GOTO 150
      END IF
      IF (L.NE.(3-1)) THEN
         L = L+1
         GOTO 140
      END IF
      IF (IND.EQ.1) THEN
         IND = 0
         GOTO 130
      END IF
C----
C---- COMPARE THRESHOLD WITH FINAL NORM
C----
      IF (THR.GT.ANRMX) GOTO 120
C----
C---- SORT EIGENVALUES AND EIGENVECTORS
C----
  290 IQ = -3
      DO 320 I = 1,3
         IQ = IQ+3
         LL = I+(I*I-I)/2
         JQ = 3*(I-2)
         DO 310 J = I,3
            JQ = JQ+3
            MM = J+(J*J-J)/2
            IF (A(LL).LT.A(MM)) THEN
               X = A(LL)
               A(LL) = A(MM)
               A(MM) = X

               ILR = IQ+1
               IMR = JQ+1
               X = R(ILR)
               R(ILR) = R(IMR)
               R(IMR) = X

               ILR = IQ+2
               IMR = JQ+2
               X = R(ILR)
               R(ILR) = R(IMR)
               R(IMR) = X

               ILR = IQ+3
               IMR = JQ+3
               X = R(ILR)
               R(ILR) = R(IMR)
               R(IMR) = X
            END IF
  310    CONTINUE
  320 CONTINUE

      RETURN
      END

      INTEGER FUNCTION GVFCBT(BOOLS,NUM)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- COUNTS HOW MANY BOOLS IN LOGICAL ARRAY BOOLS ARE TRUE.                ----
C----                                                                       ----
C---- DO NOT FORGET TO DECLARE GVFCBT AN INTEGER IN THE CALLING MODULE.     ----
C----                                                                       ----
C---- FATAL ERRORS OCCUR IF NUM IS LARGER THAN THE DEFINED LENGTH OF BOOLS. ----
C---- CHANCES ARE THAT THESE FATAL ERRORS ARE NOT REPORTED TO THE USER.     ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT    NONE
      INTEGER     NUM, I
      LOGICAL     BOOLS(NUM)
C----
C---- INITIALIZE
C----
      GVFCBT=0
C----
C---- SEE IF SOMETHING IS TO BE DONE
C----
      IF (NUM.GT.0) THEN
C-------
C------- IF NUM IS POSITIVE, PERFORM THE ACTION
C-------
         GVFCBT=0
         DO 10 I=1,NUM
            IF (BOOLS(I)) GVFCBT=GVFCBT+1
   10    CONTINUE
         RETURN
      ELSE IF (NUM.EQ.0) THEN
C-------
C------- IF NUM IS ZERO, SOMETHING COULD BE WRONG, SO WARN THE USER
C-------
C------- BERT EDITS HERE
C-------
         WRITE (6,*) '< ZERO ATOMS SELECTED FOR FITTING'
      ELSE IF (NUM.LT.0) THEN
C-------
C------- IF NUM IS NEGATIVE, SOMETHING IS WRONG, SO WARN THE USER
C-------
C------- BERT EDITS HERE
C-------
         WRITE (6,*) '> WRONG NUMBER OF ATOMS FOR FITTING'
      END IF

      RETURN
      END
      REAL FUNCTION GVFSQR(ARGMNT)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- CALCULATES THE SQRT OF ARGMNT. THIS IS A SPECIAL FUNCTION AS SQRT     ----
C---- GETS SOMETIMES VALUES LIKE 0.0000000001 AGAINST WHICH NORMAL FORTRAN  ----
C---- DOES NOT PROTECT.                                                     ----
C---- ARGMNT IS A REAL.                                                     ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT     NONE
      REAL         ARGMNT

      IF (ARGMNT.LT.1.E-20) THEN
         GVFSQR=0.0
      ELSE
         GVFSQR=SQRT(ARGMNT)
      END IF

      RETURN
      END
